@@ -1,7 +1,7 @@
 # flake8: noqa: E402
 import os
 import time
-from typing import Optional
+from typing import Optional, List
 
 from flask import Flask, g, request
 from flask.cli import load_dotenv
@@ -14,39 +14,50 @@ from redis import Redis
 from rq import Queue
 
 
-def create(env: Optional[str] = None, path_to_config: Optional[str] = None) -> Flask:
+def create(
+    env: Optional[str] = None,
+    path_to_config: Optional[str] = None,
+    plugins: Optional[List[str]] = None,
+) -> Flask:
     """
     Create a Flask app and configure it.
 
     Set the environment by setting FLASK_ENV as environment variable (also possible in .env).
     Or, overwrite any FLASK_ENV setting by passing an env in directly (useful for testing for instance).
 
-    A path to a config file can be passed in (otherwise a config file will be searched in the home or instance directories)
+    A path to a config file can be passed in (otherwise a config file will be searched in the home or instance directories).
+
+    Also, a list of plugins can be set. Usually this works as a config setting, but this is useful for automated testing.
     """
 
     from flexmeasures.utils import config_defaults
     from flexmeasures.utils.config_utils import read_config, configure_logging
-    from flexmeasures.utils.app_utils import set_secret_key
+    from flexmeasures.utils.app_utils import set_secret_key, init_sentry
     from flexmeasures.utils.error_utils import add_basic_error_handlers
 
     # Create app
 
-    configure_logging()  # do this first, see http://flask.pocoo.org/docs/dev/logging/
+    configure_logging()  # do this first, see https://flask.palletsprojects.com/en/2.0.x/logging
     # we're loading dotenv files manually & early (can do Flask.run(load_dotenv=False)),
     # as we need to know the ENV now (for it to be recognised by Flask()).
     load_dotenv()
     app = Flask("flexmeasures")
+
     if env is not None:  # overwrite
         app.env = env
-        if env == "testing":
-            app.testing = True
-        if env == "development":
-            app.debug = config_defaults.DevelopmentConfig.DEBUG
+    if app.env == "testing":
+        app.testing = True
+    if app.env == "development":
+        app.debug = config_defaults.DevelopmentConfig.DEBUG
 
     # App configuration
 
-    read_config(app, path_to_config=path_to_config)
+    read_config(app, custom_path_to_config=path_to_config)
+    if plugins:
+        app.config["FLEXMEASURES_PLUGINS"] += plugins
     add_basic_error_handlers(app)
+    if not app.env in ("development", "documentation") and not app.testing:
+        init_sentry(app)
 
     app.mail = Mail(app)
     FlaskJSON(app)
@@ -79,10 +90,9 @@ def create(env: Optional[str] = None, path_to_config: Optional[str] = None) -> F
 
     # Some basic security measures
 
-    if not app.env == "documentation":
-        set_secret_key(app)
-        if app.config.get("SECURITY_PASSWORD_SALT", None) is None:
-            app.config["SECURITY_PASSWORD_SALT"] = app.config["SECRET_KEY"]
+    set_secret_key(app)
+    if app.config.get("SECURITY_PASSWORD_SALT", None) is None:
+        app.config["SECURITY_PASSWORD_SALT"] = app.config["SECRET_KEY"]
     if not app.env in ("documentation", "development"):
         SSLify(app)
 
@@ -92,23 +102,42 @@ def create(env: Optional[str] = None, path_to_config: Optional[str] = None) -> F
 
     register_db_at(app)
 
+    # add auth policy
+
+    from flexmeasures.auth import register_at as register_auth_at
+
+    register_auth_at(app)
+
+    # Register the CLI
+
+    from flexmeasures.cli import register_at as register_cli_at
+
+    register_cli_at(app)
+
     # Register the API
 
     from flexmeasures.api import register_at as register_api_at
 
     register_api_at(app)
 
+    # Register plugins
+    # If plugins register routes, they'll have precedence over standard UI
+    # routes (first registration wins). However, we want to control "/" separately.
+
+    from flexmeasures.utils.app_utils import root_dispatcher
+    from flexmeasures.utils.plugin_utils import register_plugins
+
+    app.add_url_rule("/", view_func=root_dispatcher)
+    register_plugins(app)
+
     # Register the UI
 
     from flexmeasures.ui import register_at as register_ui_at
 
     register_ui_at(app)
 
-    from flexmeasures.utils.app_utils import register_plugins
-
-    register_plugins(app)
-
     # Profile endpoints (if needed, e.g. during development)
+
     @app.before_request
     def before_request():
         if app.config.get("FLEXMEASURES_PROFILE_REQUESTS", False):
