@@ -1,11 +1,11 @@
 """CLI Tasks for populating the database - most useful in development"""
+from __future__ import annotations
 
 from datetime import datetime, timedelta
 from typing import Dict, List, Optional, Tuple
 import json
 
 from marshmallow import validate
-import numpy as np
 import pandas as pd
 import pytz
 from flask import current_app as app
@@ -18,6 +18,7 @@ import timely_beliefs as tb
 import timely_beliefs.utils as tb_utils
 from workalendar.registry import registry as workalendar_registry
 
+from flexmeasures.cli.utils import DeprecatedDefaultGroup
 from flexmeasures.data import db
 from flexmeasures.data.scripts.data_gen import (
     add_transmission_zone_asset,
@@ -37,7 +38,13 @@ from flexmeasures.data.models.validation_utils import (
     MissingAttributeException,
 )
 from flexmeasures.data.models.annotations import Annotation, get_or_create_annotation
-from flexmeasures.data.schemas import AwareDateTimeField, DurationField, SensorIdField
+from flexmeasures.data.schemas import (
+    AwareDateTimeField,
+    DurationField,
+    LatitudeField,
+    LongitudeField,
+    SensorIdField,
+)
 from flexmeasures.data.schemas.sensors import SensorSchema
 from flexmeasures.data.schemas.units import QuantityField
 from flexmeasures.data.schemas.generic_assets import (
@@ -241,12 +248,12 @@ def add_asset_type(**args):
 @click.option("--name", required=True)
 @click.option(
     "--latitude",
-    type=float,
+    type=LatitudeField(),
     help="Latitude of the asset's location",
 )
 @click.option(
     "--longitude",
-    type=float,
+    type=LongitudeField(),
     help="Longitude of the asset's location",
 )
 @click.option("--account-id", type=int, required=True)
@@ -279,8 +286,9 @@ def add_initial_structure():
 @click.argument("file", type=click.Path(exists=True))
 @click.option(
     "--sensor-id",
+    "sensor",
     required=True,
-    type=click.IntRange(min=1),
+    type=SensorIdField(),
     help="Sensor to which the beliefs pertain.",
 )
 @click.option(
@@ -294,6 +302,8 @@ def add_initial_structure():
     required=False,
     type=str,
     help="Unit of the data, for conversion to the sensor unit, if possible (a string unit such as 'kW' or 'm³/h').\n"
+    "Measurements of time itself that are formatted as a 'datetime' or 'timedelta' can be converted to a sensor unit representing time (such as 's' or 'h'),\n"
+    "where datetimes are represented as a duration with respect to the UNIX epoch."
     "Hint: to switch the sign of the data, prepend a minus sign.\n"
     "For example, when assigning kW consumption data to a kW production sensor, use '-kW'.",
 )
@@ -334,6 +344,12 @@ def add_initial_structure():
     multiple=True,
     help="Additional strings to recognize as NaN values. This argument can be given multiple times.",
 )
+@click.option(
+    "--keep-default-na",
+    default=False,
+    type=bool,
+    help="Whether or not to keep NaN values in the data.",
+)
 @click.option(
     "--nrows",
     required=False,
@@ -354,6 +370,30 @@ def add_initial_structure():
     type=int,
     help="Column number with values (1 is 2nd column, the default)",
 )
+@click.option(
+    "--beliefcol",
+    required=False,
+    type=int,
+    help="Column number with datetimes",
+)
+@click.option(
+    "--timezone",
+    required=False,
+    default=None,
+    help="timezone as string, e.g. 'UTC' or 'Europe/Amsterdam'",
+)
+@click.option(
+    "--filter-column",
+    "filter_columns",
+    multiple=True,
+    help="Set a column number to filter data. Use together with --filter-value.",
+)
+@click.option(
+    "--filter-value",
+    "filter_values",
+    multiple=True,
+    help="Set a column value to filter data. Only rows with this value will be added. Use together with --filter-column.",
+)
 @click.option(
     "--delimiter",
     required=True,
@@ -383,18 +423,23 @@ def add_initial_structure():
 )
 def add_beliefs(
     file: str,
-    sensor_id: int,
+    sensor: Sensor,
     source: str,
+    filter_columns: List[int],
+    filter_values: List[int],
     unit: Optional[str] = None,
     horizon: Optional[int] = None,
     cp: Optional[float] = None,
     resample: bool = True,
     allow_overwrite: bool = False,
     skiprows: int = 1,
-    na_values: List[str] = None,
+    na_values: list[str] | None = None,
+    keep_default_na: bool = False,
     nrows: Optional[int] = None,
     datecol: int = 0,
     valuecol: int = 1,
+    beliefcol: Optional[int] = None,
+    timezone: Optional[str] = None,
     delimiter: str = ",",
     decimal: str = ".",
     thousands: Optional[str] = None,
@@ -416,20 +461,10 @@ def add_beliefs(
         2020-12-03 14:10,215.6
         2020-12-03 14:20,203.8
 
-    In case no --horizon is specified, the moment of executing this CLI command is taken
-    as the time at which the beliefs were recorded.
+    In case no --horizon is specified and no beliefcol is specified,
+    the moment of executing this CLI command is taken as the time at which the beliefs were recorded.
     """
-    sensor = Sensor.query.filter(Sensor.id == sensor_id).one_or_none()
-    if sensor is None:
-        print(f"Failed to create beliefs: no sensor found with ID {sensor_id}.")
-        return
-    if source.isdigit():
-        _source = get_source_or_none(int(source), source_type="CLI script")
-        if not _source:
-            print(f"Failed to find source {source}.")
-            return
-    else:
-        _source = get_or_create_source(source, source_type="CLI script")
+    _source = parse_source(source)
 
     # Set up optional parameters for read_csv
     if file.split(".")[-1].lower() == "csv":
@@ -441,9 +476,17 @@ def add_beliefs(
         kwargs["sheet_name"] = sheet_number
     if horizon is not None:
         kwargs["belief_horizon"] = timedelta(minutes=horizon)
-    else:
+    elif beliefcol is None:
         kwargs["belief_time"] = server_now().astimezone(pytz.timezone(sensor.timezone))
 
+    # Set up optional filters:
+    if len(filter_columns) != len(filter_values):
+        raise ValueError(
+            "The number of filter columns and filter values should be the same."
+        )
+    filter_by_column = (
+        dict(zip(filter_columns, filter_values)) if filter_columns else None
+    )
     bdf = tb.read_csv(
         file,
         sensor,
@@ -453,11 +496,21 @@ def add_beliefs(
         header=None,
         skiprows=skiprows,
         nrows=nrows,
-        usecols=[datecol, valuecol],
+        usecols=[datecol, valuecol]
+        if beliefcol is None
+        else [datecol, beliefcol, valuecol],
         parse_dates=True,
         na_values=na_values,
+        keep_default_na=keep_default_na,
+        timezone=timezone,
+        filter_by_column=filter_by_column,
         **kwargs,
     )
+    duplicate_rows = bdf.index.duplicated(keep="first")
+    if any(duplicate_rows) > 0:
+        print("Duplicates found. Dropping duplicates for the following records:")
+        print(bdf[duplicate_rows])
+        bdf = bdf[~duplicate_rows]
     if unit is not None:
         bdf["event_value"] = convert_units(
             bdf["event_value"],
@@ -769,14 +822,37 @@ def create_forecasts(
         )
 
 
-@fm_add_data.command("schedule")
+# todo: repurpose `flexmeasures add schedule` (deprecated since v0.12),
+#       - see https://github.com/FlexMeasures/flexmeasures/pull/537#discussion_r1048680231
+#       - hint for repurposing to invoke custom logic instead of a default subcommand:
+#             @fm_add_data.group("schedule", invoke_without_command=True)
+#             def create_schedule():
+#                 if ctx.invoked_subcommand:
+#                     ...
+@fm_add_data.group(
+    "schedule",
+    cls=DeprecatedDefaultGroup,
+    default="storage",
+    deprecation_message="The command 'flexmeasures add schedule' is deprecated. Please use `flexmeasures add schedule for-storage` instead.",
+)
+@click.pass_context
+@with_appcontext
+def create_schedule(ctx):
+    """(Deprecated) Create a new schedule for a given power sensor.
+
+    THIS COMMAND HAS BEEN RENAMED TO `flexmeasures add schedule for-storage`
+    """
+    pass
+
+
+@create_schedule.command("for-storage")
 @with_appcontext
 @click.option(
     "--sensor-id",
     "power_sensor",
     type=SensorIdField(),
     required=True,
-    help="Create schedule for this sensor. Follow up with the sensor's ID.",
+    help="Create schedule for this sensor. Should be a power sensor. Follow up with the sensor's ID.",
 )
 @click.option(
     "--consumption-price-sensor",
@@ -860,7 +936,7 @@ def create_forecasts(
     help="Whether to queue a scheduling job instead of computing directly. "
     "To process the job, run a worker (on any computer, but configured to the same databases) to process the 'scheduling' queue. Defaults to False.",
 )
-def create_schedule(
+def add_schedule_for_storage(
     power_sensor: Sensor,
     consumption_price_sensor: Sensor,
     production_price_sensor: Sensor,
@@ -874,14 +950,13 @@ def create_schedule(
     roundtrip_efficiency: Optional[ur.Quantity] = None,
     as_job: bool = False,
 ):
-    """Create a new schedule for a given power sensor.
+    """Create a new schedule for a storage asset.
 
     Current limitations:
 
-    - only supports battery assets and Charge Points
-    - only supports datetimes on the hour or a multiple of the sensor resolution thereafter
+    - Limited to power sensors (probably possible to generalize to non-electric assets)
+    - Only supports datetimes on the hour or a multiple of the sensor resolution thereafter
     """
-
     # todo: deprecate the 'optimization-context-id' argument in favor of 'consumption-price-sensor' (announced v0.11.0)
     tb_utils.replace_deprecated_argument(
         "optimization-context-id",
@@ -890,38 +965,23 @@ def create_schedule(
         consumption_price_sensor,
     )
 
-    # Parse input
+    # Parse input and required sensor attributes
     if not power_sensor.measures_power:
         click.echo(f"Sensor with ID {power_sensor.id} is not a power sensor.")
         raise click.Abort()
     if production_price_sensor is None:
         production_price_sensor = consumption_price_sensor
     end = start + duration
-    for attribute in ("min_soc_in_mwh", "max_soc_in_mwh"):
-        try:
-            check_required_attributes(power_sensor, [(attribute, float)])
-        except MissingAttributeException:
-            click.echo(f"{power_sensor} has no {attribute} attribute.")
-            raise click.Abort()
-    soc_targets = pd.Series(
-        np.nan,
-        index=pd.date_range(
-            pd.Timestamp(start).tz_convert(power_sensor.timezone),
-            pd.Timestamp(end).tz_convert(power_sensor.timezone),
-            freq=power_sensor.event_resolution,
-            closed="right",
-        ),  # note that target values are indexed by their due date (i.e. closed="right")
-    )
 
-    # Convert round-trip efficiency to dimensionless
-    if roundtrip_efficiency is not None:
-        roundtrip_efficiency = roundtrip_efficiency.to(
-            ur.Quantity("dimensionless")
-        ).magnitude
-
-    # Convert SoC units to MWh, given the storage capacity
+    # Convert SoC units (we ask for % in this CLI) to MWh, given the storage capacity
+    try:
+        check_required_attributes(power_sensor, [("max_soc_in_mwh", float)])
+    except MissingAttributeException:
+        click.echo(f"Sensor {power_sensor} has no max_soc_in_mwh attribute.")
+        raise click.Abort()
     capacity_str = f"{power_sensor.get_attribute('max_soc_in_mwh')} MWh"
     soc_at_start = convert_units(soc_at_start.magnitude, soc_at_start.units, "MWh", capacity=capacity_str)  # type: ignore
+    soc_targets = []
     for soc_target_tuple in soc_target_strings:
         soc_target_value_str, soc_target_dt_str = soc_target_tuple
         soc_target_value = convert_units(
@@ -931,44 +991,39 @@ def create_schedule(
             capacity=capacity_str,
         )
         soc_target_datetime = pd.Timestamp(soc_target_dt_str)
-        soc_targets.loc[soc_target_datetime] = soc_target_value
+        soc_targets.append(dict(value=soc_target_value, datetime=soc_target_datetime))
     if soc_min is not None:
         soc_min = convert_units(soc_min.magnitude, str(soc_min.units), "MWh", capacity=capacity_str)  # type: ignore
     if soc_max is not None:
         soc_max = convert_units(soc_max.magnitude, str(soc_max.units), "MWh", capacity=capacity_str)  # type: ignore
+    if roundtrip_efficiency is not None:
+        roundtrip_efficiency = roundtrip_efficiency.magnitude / 100.0
 
+    scheduling_kwargs = dict(
+        sensor=power_sensor,
+        start=start,
+        end=end,
+        belief_time=server_now(),
+        resolution=power_sensor.event_resolution,
+        flex_model={
+            "soc-at-start": soc_at_start,
+            "soc-targets": soc_targets,
+            "soc-min": soc_min,
+            "soc-max": soc_max,
+            "soc-unit": "MWh",
+            "roundtrip-efficiency": roundtrip_efficiency,
+        },
+        flex_context={
+            "consumption-price-sensor": consumption_price_sensor.id,
+            "production-price-sensor": production_price_sensor.id,
+        },
+    )
     if as_job:
-        job = create_scheduling_job(
-            sensor_id=power_sensor.id,
-            start_of_schedule=start,
-            end_of_schedule=end,
-            belief_time=server_now(),
-            resolution=power_sensor.event_resolution,
-            soc_at_start=soc_at_start,
-            soc_targets=soc_targets,
-            soc_min=soc_min,
-            soc_max=soc_max,
-            roundtrip_efficiency=roundtrip_efficiency,
-            consumption_price_sensor=consumption_price_sensor,
-            production_price_sensor=production_price_sensor,
-        )
+        job = create_scheduling_job(**scheduling_kwargs)
         if job:
             print(f"New scheduling job {job.id} has been added to the queue.")
     else:
-        success = make_schedule(
-            sensor_id=power_sensor.id,
-            start=start,
-            end=end,
-            belief_time=server_now(),
-            resolution=power_sensor.event_resolution,
-            soc_at_start=soc_at_start,
-            soc_targets=soc_targets,
-            soc_min=soc_min,
-            soc_max=soc_max,
-            roundtrip_efficiency=roundtrip_efficiency,
-            consumption_price_sensor=consumption_price_sensor,
-            production_price_sensor=production_price_sensor,
-        )
+        success = make_schedule(**scheduling_kwargs)
         if success:
             print("New schedule is stored.")
 
@@ -1021,7 +1076,9 @@ def add_toy_account(kind: str, name: str):
             db.session.add(asset)
             if asset_type == "battery":
                 asset.attributes = dict(
-                    capacity_in_mw=0.5, min_soc_in_mwh=0.05, max_soc_in_mwh=0.45
+                    capacity_in_mw=0.5,
+                    min_soc_in_mwh=0.05,
+                    max_soc_in_mwh=0.45,
                 )
                 # add charging sensor to battery
                 charging_sensor = Sensor(
@@ -1052,13 +1109,24 @@ def add_toy_account(kind: str, name: str):
             )
         db.session.add(day_ahead_sensor)
 
+        # add day-ahead sensor to battery page
+        db.session.flush()
+        battery = charging_sensor.generic_asset
+        battery.attributes["sensors_to_show"] = [
+            day_ahead_sensor.id,
+            charging_sensor.id,
+        ]
     db.session.commit()
 
     click.echo(
         f"Toy account {name} with user {user.email} created successfully. You might want to run `flexmeasures show account --id {user.account.id}`"
     )
-    click.echo(f"The sensor for battery charging is {charging_sensor}.")
-    click.echo(f"The sensor for Day ahead prices is {day_ahead_sensor}.")
+    click.echo(
+        f"The sensor for battery discharging is {charging_sensor} (ID: {charging_sensor.id})."
+    )
+    click.echo(
+        f"The sensor for Day ahead prices is {day_ahead_sensor} (ID: {day_ahead_sensor.id})."
+    )
 
 
 app.cli.add_command(fm_add_data)
@@ -1078,3 +1146,14 @@ def check_errors(errors: Dict[str, List[str]]):
             f"Please correct the following errors:\n{errors}.\n Use the --help flag to learn more."
         )
         raise click.Abort
+
+
+def parse_source(source):
+    if source.isdigit():
+        _source = get_source_or_none(int(source))
+        if not _source:
+            print(f"Failed to find source {source}.")
+            return
+    else:
+        _source = get_or_create_source(source, source_type="CLI script")
+    return _source
