@@ -20,7 +20,10 @@ from flexmeasures.data.models.user import User
 from flexmeasures.data.queries.annotations import query_asset_annotations
 from flexmeasures.auth.policy import AuthModelMixin, EVERY_LOGGED_IN_USER
 from flexmeasures.utils import geo_utils
-from flexmeasures.utils.time_utils import server_now
+from flexmeasures.utils.time_utils import (
+    determine_minimum_resampling_resolution,
+    server_now,
+)
 
 
 class GenericAssetType(db.Model):
@@ -342,6 +345,7 @@ class GenericAsset(db.Model, AuthModelMixin):
         source: Optional[
             Union[DataSource, List[DataSource], int, List[int], str, List[str]]
         ] = None,
+        most_recent_beliefs_only: bool = True,
         most_recent_events_only: bool = False,
         as_json: bool = False,
     ) -> Union[BeliefsDataFrame, str]:
@@ -373,7 +377,7 @@ class GenericAsset(db.Model, AuthModelMixin):
                 horizons_at_least=horizons_at_least,
                 horizons_at_most=horizons_at_most,
                 source=source,
-                most_recent_beliefs_only=True,
+                most_recent_beliefs_only=most_recent_beliefs_only,
                 most_recent_events_only=most_recent_events_only,
                 one_deterministic_belief_per_event_per_source=True,
             )
@@ -381,31 +385,44 @@ class GenericAsset(db.Model, AuthModelMixin):
             from flexmeasures.data.services.time_series import simplify_index
 
             if sensors:
-                condition = list(
-                    bdf.event_resolution
-                    for bdf in bdf_dict.values()
-                    if bdf.event_resolution > timedelta(0)
-                )
-                minimum_non_zero_resolution = (
-                    min(condition) if any(condition) else timedelta(0)
+                minimum_resampling_resolution = determine_minimum_resampling_resolution(
+                    [bdf.event_resolution for bdf in bdf_dict.values()]
                 )
                 df_dict = {}
                 for sensor, bdf in bdf_dict.items():
                     if bdf.event_resolution > timedelta(0):
-                        bdf = bdf.resample_events(minimum_non_zero_resolution)
+                        bdf = bdf.resample_events(minimum_resampling_resolution)
+                    bdf["belief_horizon"] = bdf.belief_horizons.to_numpy()
                     df = simplify_index(
-                        bdf, index_levels_to_columns=["source"]
-                    ).set_index(["source"], append=True)
-                    df_dict[sensor.id] = df.rename(columns=dict(event_value=sensor.id))
-                df = list(df_dict.values())[0].join(
-                    list(df_dict.values())[1:], how="outer"
-                )
+                        bdf,
+                        index_levels_to_columns=["source"]
+                        if most_recent_beliefs_only
+                        else ["belief_time", "source"],
+                    ).set_index(
+                        ["source"]
+                        if most_recent_beliefs_only
+                        else ["belief_time", "source"],
+                        append=True,
+                    )
+                    df["sensor"] = sensor  # or some JSONifiable representation
+                    df = df.set_index(["sensor"], append=True)
+                    df_dict[sensor.id] = df
+                df = pd.concat(df_dict.values())
             else:
                 df = simplify_index(
-                    BeliefsDataFrame(), index_levels_to_columns=["source"]
-                ).set_index(["source"], append=True)
+                    BeliefsDataFrame(),
+                    index_levels_to_columns=["source"]
+                    if most_recent_beliefs_only
+                    else ["belief_time", "source"],
+                ).set_index(
+                    ["source"]
+                    if most_recent_beliefs_only
+                    else ["belief_time", "source"],
+                    append=True,
+                )
             df = df.reset_index()
             df["source"] = df["source"].apply(lambda x: x.to_dict())
+            df["sensor"] = df["sensor"].apply(lambda x: x.to_dict())
             return df.to_json(orient="records")
         return bdf_dict
 
@@ -413,18 +430,27 @@ class GenericAsset(db.Model, AuthModelMixin):
     def sensors_to_show(self) -> List["Sensor"]:  # noqa F821
         """Sensors to show, as defined by the sensors_to_show attribute.
 
+        Sensors to show are defined as a list of sensor ids, which
+        is set by the "sensors_to_show" field of the asset's "attributes" column.
+        Valid sensors either belong to the asset itself, to other assets in the same account,
+        or to public assets.
+
+
         Defaults to two of the asset's sensors.
         """
         if not self.has_attribute("sensors_to_show"):
             return self.sensors[:2]
 
-        from flexmeasures.data.services.sensors import get_public_sensors
+        from flexmeasures.data.services.sensors import get_sensors
 
         sensor_ids = self.get_attribute("sensors_to_show")
         sensor_map = {
             sensor.id: sensor
-            for sensor in self.sensors + get_public_sensors(sensor_ids)
-            if sensor.id in sensor_ids
+            for sensor in get_sensors(
+                account=self.owner,
+                include_public_assets=True,
+                sensor_id_allowlist=sensor_ids,
+            )
         }
 
         # Return sensors in the order given by the sensors_to_show attribute
