@@ -11,7 +11,7 @@ from flexmeasures.data.queries.utils import (
 )
 from flexmeasures.data.services.time_series import set_bdf_source
 from flexmeasures.utils import calculations, time_utils
-from flexmeasures.data.services.resources import Resource, find_closest_sensor
+from flexmeasures.data.services.resources import Resource
 from flexmeasures.data.models.assets import Asset, Power
 from flexmeasures.data.models.time_series import Sensor, TimedBelief
 from flexmeasures.data.models.weather import WeatherSensorType
@@ -258,7 +258,12 @@ def get_weather_data(
     if sensor_type:
         # Find the 50 closest weather sensors
         sensor_type_name = sensor_type.name
-        closest_sensors = find_closest_sensor(sensor_type_name, n=50, object=asset)
+        closest_sensors = Sensor.find_closest(
+            generic_asset_type_name=asset.generic_asset.generic_asset_type.name,
+            sensor_name=sensor_type_name,
+            n=50,
+            object=asset,
+        )
         if closest_sensors:
             closest_sensor = closest_sensors[0]
 
