@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 import re
 from datetime import datetime, timedelta
 from typing import List, Union, Tuple, Optional
@@ -12,8 +14,8 @@ from dateutil import tz
 
 
 def server_now() -> datetime:
-    """The current time of the FlexMeasures platform. UTC time, localized to the FlexMeasures timezone."""
-    return as_server_time(datetime.utcnow())
+    """The current time (timezone aware), converted to the timezone of the FlexMeasures platform."""
+    return get_timezone().fromutc(datetime.utcnow())
 
 
 def ensure_local_timezone(
@@ -199,7 +201,9 @@ def get_most_recent_hour() -> datetime:
 
 
 def get_most_recent_clocktime_window(
-    window_size_in_minutes: int, now: Optional[datetime] = None
+    window_size_in_minutes: int,
+    now: Optional[datetime] = None,
+    grace_period_in_seconds: Optional[int] = 0,
 ) -> Tuple[datetime, datetime]:
     """
     Calculate a recent time window, returning a start and end minute so that
@@ -208,6 +212,9 @@ def get_most_recent_clocktime_window(
     Calling this function at 15:01:xx with window size 5 -> (14:55:00, 15:00:00)
     Calling this function at 03:36:xx with window size 15 -> (03:15:00, 03:30:00)
 
+    We can demand a grace period (of x seconds) to have passed before we are ready to accept that we're in a new window:
+    Calling this function at 15:00:16 with window size 5 and grace period of 30 seconds -> (14:50:00, 14:55:00)
+
     window_size_in_minutes is assumed to > 0 and < = 60, and a divisor of 60 (1, 2, ..., 30, 60).
 
     If now is not given, the current server time is used.
@@ -221,7 +228,13 @@ def get_most_recent_clocktime_window(
     assert 60 % window_size_in_minutes == 0
     if now is None:
         now = server_now()
-    last_full_minute = now.replace(second=0, microsecond=0) - timedelta(minutes=1)
+    last_full_minute = now.replace(second=0, microsecond=0)
+    if (
+        grace_period_in_seconds is not None
+        and grace_period_in_seconds > 0
+        and (now - last_full_minute).seconds < grace_period_in_seconds
+    ):
+        last_full_minute -= timedelta(minutes=1 + grace_period_in_seconds // 60)
     last_round_minute = last_full_minute.minute - (
         last_full_minute.minute % window_size_in_minutes
     )
@@ -322,3 +335,28 @@ def duration_isoformat(duration: timedelta):
     # at least one component has to be there.
     repl = ret and "".join(ret) or "T0H"
     return re.sub("%P", repl, "P%P")
+
+
+def determine_minimum_resampling_resolution(
+    event_resolutions: list[timedelta],
+) -> timedelta:
+    """Return minimum non-zero event resolution, or zero resolution if none of the event resolutions is non-zero."""
+    condition = list(
+        event_resolution
+        for event_resolution in event_resolutions
+        if event_resolution > timedelta(0)
+    )
+    return min(condition) if any(condition) else timedelta(0)
+
+
+def to_http_time(dt: pd.Timestamp | datetime) -> str:
+    """Formats datetime using the Internet Message Format fixdate.
+
+    >>> to_http_time(pd.Timestamp("2022-12-13 14:06:23Z"))
+    Tue, 13 Dec 2022 14:06:23 GMT
+
+    References
+    ----------
+    IMF-fixdate: https://www.rfc-editor.org/rfc/rfc7231#section-7.1.1.1
+    """
+    return dt.strftime("%a, %d %b %Y %H:%M:%S GMT")
